package com.sdk.sdkdemo;

import android.app.Presentation;
import android.content.Context;
import android.content.res.AssetManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.Display;

import com.youth.banner.Banner;
import com.youth.banner.BannerConfig;
import com.youth.banner.Transformer;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Administrator
 *
 * @author 猿史森林
 *         Date: 2019/9/21
 *         Class description:Customer show
 */
public class Customer extends Presentation {

    private Banner banner;
    private AssetManager assetManager;
    private List<Bitmap> bitmapList;
    private Handler mainHandler = new Handler(Looper.getMainLooper());


    public Customer(Context outerContext, Display display) {
        super(outerContext, display);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        setContentView(R.layout.customer);
        banner = findViewById(R.id.banner);
        banner.setImageLoader(new CustomerImageLoader());
        banner.setDelayTime(4000);
        banner.isAutoPlay(true);
        banner.setBannerAnimation(Transformer.Default);
        banner.setBannerStyle(BannerConfig.CIRCLE_INDICATOR);
        banner.setIndicatorGravity(BannerConfig.CENTER);


        assetManager = getContext().getAssets();
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    String[] images = assetManager.list("");
                    bitmapList = new ArrayList<>();
                    for (String path : images) {
                        if (path.endsWith(".png") || path.endsWith(".jpg")) {
                            InputStream inputStream = assetManager.open(path);
                            bitmapList.add(BitmapFactory.decodeStream(inputStream));
                            inputStream.close();
                        }
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                } finally {
                    assetManager.close();
                }
                mainHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        banner.setImages(bitmapList);
                        banner.start();
                    }
                });
            }
        }).start();

    }

    @Override
    protected void onStart() {
        super.onStart();
        banner.startAutoPlay();
    }

    @Override
    protected void onStop() {
        super.onStop();
        banner.stopAutoPlay();
    }
}
