package com.sdk.sdkdemo;

import androidx.appcompat.app.AppCompatActivity;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.hardware.display.DisplayManager;
import android.os.Bundle;
import android.text.TextPaint;
import android.util.Log;
import android.view.Display;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;
import android.widget.Toast;

import com.sdklib.Observer;
import com.sdklib.command.Command;
import com.sdklib.command.EscCommand;
import com.sdklib.connection.CashDrawerConnection;

import java.io.IOException;
import java.text.DecimalFormat;
import java.util.Random;

import static android.graphics.Bitmap.createBitmap;

/**
 * Created by Administrator
 *
 * @author 猿史森林
 * Date: 2019/9/21
 * Class description:
 */
public class MainActivity extends AppCompatActivity {

    private static final String TAG = MainActivity.class.getSimpleName();
    /**
     * ESC Query printer real-time status.Out of paper status
     */
    private static final int ESC_STATE_PAPER_ERR = 0x20;

    /**
     * ESC Query printer real-time status.Cover open status
     */
    private static final int ESC_STATE_COVER_OPEN = 0x04;

    /**
     * ESC Query printer real-time status.Printer error
     */
    private static final int ESC_STATE_ERR_OCCURS = 0x40;
    private EscCommand escCommand;
    private CashDrawerConnection drawerConnection;
    private DisplayManager displayManager;
    private Customer customer;
    private boolean startOrEndSecondaryScreen;
    private Button btnCustomer;
    private ThreadFactoryBuilder threadFactoryBuilder = new ThreadFactoryBuilder("main_activity");

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_main);

        displayManager = (DisplayManager) getSystemService(DISPLAY_SERVICE);
        //bind cash service
        Intent intent = new Intent("com.android.settings.EasyService");
        intent.setPackage("com.android.settings");
        drawerConnection = new CashDrawerConnection();
        bindService(intent, drawerConnection, Context.BIND_AUTO_CREATE);

        btnCustomer = findViewById(R.id.customer);
        btnCustomer.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (!startOrEndSecondaryScreen) {
                    showSecondaryScreen();
                    startOrEndSecondaryScreen = true;
                    btnCustomer.setText(R.string.str_close_customer);
                } else {
                    startOrEndSecondaryScreen = false;
                    customer.dismiss();
                    btnCustomer.setText(R.string.str_open_customer);
                }
            }
        });
    }

    /**
     * Print custom content
     *
     * @param view
     */
    public void printTest(View view) {
        ThreadPool.getInstantiation().addTask(threadFactoryBuilder.newThread(new Runnable() {
            @Override
            public void run() {
                escCommand = EscCommand.getInstance();
                if (!escCommand.isConnected()) {
                    connect(new ConnectState() {
                        @Override
                        public void success() {
                            printTest();
                        }
                    });
                } else {
                    printTest();
                }
            }
        }));
    }

    /**
     * Connect printer
     */
    private void connect(final ConnectState connecter) {
        escCommand = EscCommand.getInstance();
        try {
            escCommand.connect(MainActivity.this, new Observer() {
                @Override
                public void success() {
                    Log.i(TAG, "success");
                    connecter.success();
                }

                @Override
                public void fail() {
                    Log.e(TAG, "Connection failed");
                }

                @Override
                public void error(Throwable throwable) {
                    Log.e(TAG, "Connection error - " + throwable.toString());
                }

                @Override
                public void other(int code) {
                    Log.e(TAG, "Connection other error - " + code);
                }
            });
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private String[] menus = new String[]{"特色咖喱肠","纽伦堡风味香","美味素鸡","时蔬豆腐卷","韩国泡菜","面酱黄瓜"};
    private String[] prices = new String[]{"18.0", "25.0","10.0","10.0","10.0","10.0"};

    /**
     * Print custom content
     */
    private void printTest() {
        if (printerIsNormal()) {
            escCommand.addPrintAndFeedLines((byte) 3);
            // Set print center
            escCommand.addSelectJustification(EscCommand.JUSTIFICATION.CENTER);
            // Set to double height and width
            escCommand.addSelectPrintModes(EscCommand.FONT.FONTA, EscCommand.ENABLE.OFF, EscCommand.ENABLE.ON, EscCommand.ENABLE.ON, EscCommand.ENABLE.OFF);
            // Print text
            escCommand.addText("Sample\n");
            escCommand.addPrintAndLineFeed();

            // Cancel double height and double width
            escCommand.addSelectPrintModes(EscCommand.FONT.FONTA, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF);
            // Set print left alignment
            escCommand.addSelectJustification(EscCommand.JUSTIFICATION.LEFT);
            // Print text
            escCommand.addText("Print text\n");
            // Print text
            escCommand.addText("Welcome to use printer!\n");

            /* Printing traditional Chinese requires the printer to support traditional font library */
            String message = "打印繁體中文\n";
            // Print text
            escCommand.addText(message, "GB2312");
            escCommand.addPrintAndLineFeed();

            // Print text
            escCommand.addText("Left");
            //Set horizontal and vertical unit distance
            escCommand.addSetHorAndVerMotionUnits((byte) 7, (byte) 0);
            //Set absolute position
            escCommand.addSetAbsolutePrintPosition((short) 6);
            // Print text
            escCommand.addText("Center");
            //Set absolute position
            escCommand.addSetAbsolutePrintPosition((short) 10);
            // Print text
            escCommand.addText("Right");
            //Paper feed line
            escCommand.addPrintAndLineFeed();

            // Print text
            escCommand.addText("Print bitmap!\n");
            Bitmap b = BitmapFactory.decodeResource(getResources(),
                    R.drawable.printer);
            // Print raster bitmap
            escCommand.addRastBitImage(b, 380, 0);
            b.recycle();

            // Print text
            escCommand.addText("Print code128\n");
            // Set barcode recognizable characters(HRI)
            // Set the position of barcode recognizable characters under the barcode
            escCommand.addSelectPrintingPositionForHRICharacters(EscCommand.HRI_POSITION.BELOW);
            // Set bar code height to 60 points
            escCommand.addSetBarcodeHeight((byte) 60);
            // Set barcode unit width to 1
            escCommand.addSetBarcodeWidth((byte) 1);
            // Print code 128
            escCommand.addCODE128(escCommand.genCodeB("PRINTER"));
            escCommand.addPrintAndLineFeed();

            // Print text
            escCommand.addText("Print QRcode\n");
            // Set error correction level
            escCommand.addSelectErrorCorrectionLevelForQRCode((byte) 0x31);
            // Set QRcode module size
            escCommand.addSelectSizeOfModuleForQRCode((byte) 3);
            // Set QRCode content
            escCommand.addStoreQRCodeData("ABCD1234");
            // Print QRCode
            escCommand.addPrintQRCode();
            escCommand.addPrintAndLineFeed();

            // Set print left alignment
            escCommand.addSelectJustification(EscCommand.JUSTIFICATION.CENTER);
            //Print text
            escCommand.addText("Completed!\r\n");

            escCommand.addPrintAndFeedLines((byte) 8);
        } else {
            Log.e(TAG, "Printer is not working properly.");
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Utils.toast(MainActivity.this, "Printer is not working properly.");
                }
            });
        }
    }

    public void printerState(View view) throws IOException {
        escCommand = EscCommand.getInstance();
        if (!escCommand.isConnected()) {
            connect(new ConnectState() {
                @Override
                public void success() {
                    readPrinterState();
                }
            });
        } else {
            readPrinterState();
        }
    }

    /**
     * Receive printer data
     */
    private void readPrinterState() {
        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    Log.i(TAG, "Read data");
                    EscCommand.getInstance().addQueryPrinterStatus();
                    byte[] buffer = new byte[4];
                    int ret = EscCommand.getInstance().read(buffer, 0);
                    String status = App.getContext().getString(R.string.str_printer_conn_normal);
                    if (ret > 0) {
                        if ((buffer[0] & ESC_STATE_PAPER_ERR) > 0) {
                            status += " " + App.getContext().getString(R.string.str_printer_out_of_paper);
                        }
                        if ((buffer[0] & ESC_STATE_COVER_OPEN) > 0) {
                            status += " " + App.getContext().getString(R.string.str_printer_open_cover);
                        }
                        if ((buffer[0] & ESC_STATE_ERR_OCCURS) > 0) {
                            status += " " + App.getContext().getString(R.string.str_printer_error);
                        }
                        System.out.println(App.getContext().getString(R.string.str_state) + status);
                        final String finalStatus = status;
                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                Utils.toast(App.getContext(), finalStatus);
                            }
                        });
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        });
        thread.start();
    }

    /**
     * Open cash
     *
     * @param view
     */
    public void openCash(View view) {
        final EscCommand escCommand = EscCommand.getInstance();
        if (escCommand.openCashDrawer(drawerConnection, 1)) {
            Toast.makeText(MainActivity.this, "Cash opened successfully", Toast.LENGTH_SHORT).show();
        } else {
            Toast.makeText(MainActivity.this, "Failed to open the cash", Toast.LENGTH_SHORT).show();
        }
    }

    private void showSecondaryScreen() {
        Display[] displays = displayManager.getDisplays(DisplayManager.DISPLAY_CATEGORY_PRESENTATION);
        if (displays.length > 0) {
            Display display = displays[0];
            if (customer == null) {
                customer = new Customer(this, display);
            }
            customer.show();
            customer.setOnDismissListener(new DialogInterface.OnDismissListener() {
                @Override
                public void onDismiss(DialogInterface dialogInterface) {

                }
            });
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        unbindService(drawerConnection);
    }

    public boolean printerIsNormal() {
        escCommand = EscCommand.getInstance();
        boolean isSuccess = escCommand.addQueryPrinterStatus();
        if(isSuccess){
            byte[] buffer = new byte[4];
            try {
                int ret = escCommand.read(buffer, 2000);
                if (ret > 0) {
                    if ((buffer[0] & ESC_STATE_PAPER_ERR) == 0 && (buffer[0] & ESC_STATE_COVER_OPEN) == 0 && (buffer[0] & ESC_STATE_ERR_OCCURS) == 0) {
                        return true;
                    }
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return false;
    }
}
